@@ -56,7 +56,6 @@ typedef struct _CdCache {
 
 #ifdef HAVE_HAL
   LibHalContext *ctx;
-  gboolean has_medium;
 #endif
 #ifdef HAVE_HAL_0_5
   /* If the disc is a media, have the UDI available here */
@@ -66,13 +65,15 @@ typedef struct _CdCache {
   /* capabilities of the device */
   int cap;
 
+  /* Whether we have a medium */
+  guint has_medium : 1;
   /* if we're checking a media, or a dir */
-  gboolean is_media;
+  guint is_media : 1;
 
   /* indicates if we mounted this mountpoint ourselves or if it
    * was already mounted. */
-  gboolean self_mounted;
-  gboolean mounted;
+  guint self_mounted : 1;
+  guint mounted : 1;
 } CdCache;
 
 /* 
@@ -278,7 +279,8 @@ cd_cache_new_hal_ctx (void)
       libhal_ctx_free (ctx);
       return NULL;
     }
-    return ctx;
+    if (libhal_ctx_init (ctx, &error))
+      return ctx;
   }
 
   if (dbus_error_is_set (&error)) {
@@ -286,7 +288,6 @@ cd_cache_new_hal_ctx (void)
     dbus_error_free (&error);
   }
 
-  libhal_ctx_shutdown (ctx, NULL);
   libhal_ctx_free (ctx);
   if (conn != NULL)
     dbus_connection_unref (conn);
@@ -592,6 +593,8 @@ static MediaType
 cd_cache_disc_is_cdda (CdCache *cache,
 		       GError **error)
 {
+  MediaType type;
+
   /* We can't have audio CDs on disc, yet */
   if (cache->is_media == FALSE)
     return MEDIA_TYPE_DATA;
@@ -607,6 +610,7 @@ cd_cache_disc_is_cdda (CdCache *cache,
 
     is_cdda = libhal_device_get_property_bool (cache->ctx,
 	cache->disc_udi, "volume.disc.has_audio", &error);
+    type = is_cdda ? MEDIA_TYPE_CDDA : MEDIA_TYPE_DATA;
 
     if (dbus_error_is_set (&error)) {
       g_warning ("Error checking whether the volume is an audio CD: %s",
@@ -614,13 +618,14 @@ cd_cache_disc_is_cdda (CdCache *cache,
       dbus_error_free (&error);
       return MEDIA_TYPE_ERROR;
     }
-    return is_cdda ? MEDIA_TYPE_CDDA : MEDIA_TYPE_DATA;
+    return type;
   }
 #else
   {
-    MediaType type = MEDIA_TYPE_DATA;
     GList *vol, *item;
 
+    type = MEDIA_TYPE_DATA;
+
     for (vol = item = gnome_vfs_drive_get_mounted_volumes (cache->drive);
 	item != NULL; item = item->next) {
       char *mnt = gnome_vfs_volume_get_activation_uri (item->data);
