@@ -1324,8 +1324,11 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 		/* Either it's a URI, or it has a proper path ... */
 		if (strstr(lines[i], "://") != NULL
 				|| lines[i][0] == G_DIR_SEPARATOR) {
-			totem_pl_parser_add_one_url (parser, lines[i],
-					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
+			if (totem_pl_parser_parse_internal (parser, lines[i])
+					!= TOTEM_PL_PARSER_RESULT_SUCCESS) {
+				totem_pl_parser_add_one_url (parser, lines[i],
+						totem_pl_parser_get_extinfo_title (extinfo, lines, i));
+			}
 			extinfo = FALSE;
 		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
 			/* ... Or it's in the windows smb form
@@ -1435,6 +1438,7 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 	xmlChar *title = NULL;
 	xmlNodePtr node;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
+	xmlChar *newbase = NULL;
 
 	for (node = parent->children; node != NULL; node = node->next) {
 		if (node->name == NULL)
@@ -1443,6 +1447,13 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		if (g_ascii_strcasecmp ((char *)node->name, "title") == 0) {
 			title = xmlNodeListGetString(doc, node->children, 1);
 		}
+		if (g_ascii_strcasecmp ((char *)node->name, "base") == 0) {
+			newbase = xmlGetProp (node, (const xmlChar *)"href");
+			if (newbase == NULL)
+				newbase = xmlGetProp (node, (const xmlChar *)"HREF");
+			if (newbase != NULL)
+				base = (char *)newbase;
+		}
 
 		if (g_ascii_strcasecmp ((char *)node->name, "entry") == 0) {
 			/* Whee found an entry here, find the REF and TITLE */
@@ -1462,6 +1473,9 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		}
 	}
 
+	if (newbase)
+		xmlFree (newbase);
+
 	if (title)
 		xmlFree (title);
 
@@ -1822,12 +1836,82 @@ totem_pl_parser_dir_compare (GnomeVFSFileInfo *a, GnomeVFSFileInfo *b)
 	}
 }
 
+static char *
+totem_pl_parser_iso_get_title (const char *url)
+{
+	char *fname;
+	FILE  *file;
+#define BUFFER_SIZE 128
+	char buf [BUFFER_SIZE+1];
+	int res;
+	char *str;
+
+	fname = g_filename_from_uri (url, NULL, NULL);
+	if (fname == NULL)
+		return NULL;
+
+	file = fopen (fname, "rb");
+	if (file == NULL)
+		return NULL;
+
+	/* Verify we have an ISO image */
+	/* This check is for the raw sector images */
+	res = fseek (file, 37633L, SEEK_SET);
+	if (res != 0) {
+		fclose (file);
+		return NULL;
+	}
+
+	res = fread (buf, sizeof (char), 5, file);
+	if (res != 5 || strncmp (buf, "CD001", 5) != 0) {
+		/* Standard ISO images */
+		res = fseek (file, 32769L, SEEK_SET);
+		if (res != 0) {
+			fclose (file);
+			return NULL;
+		}
+		res = fread (buf, sizeof (char), 5, file);
+		if (res != 5 || strncmp (buf, "CD001", 5) != 0) {
+			/* High Sierra images */
+			res = fseek (file, 32776L, SEEK_SET);
+			if (res != 0) {
+				fclose (file);
+				return NULL;
+			}
+			res = fread (buf, sizeof (char), 5, file);
+			if (res != 5 || strncmp (buf, "CDROM", 5) != 0) {
+				fclose (file);
+				return NULL;
+			}
+		}
+	}
+	/* Extract the volume label from the image */
+	res = fseek (file, 32808L, SEEK_SET);
+	if (res != 0) {
+		fclose (file);
+		return NULL;
+	}
+	res = fread (buf, sizeof(char), BUFFER_SIZE, file);
+	fclose (file);
+	if (res != BUFFER_SIZE)
+		return NULL;
+
+	buf [BUFFER_SIZE] = '\0';
+	str = g_strdup (g_strstrip (buf));
+	if (!g_utf8_validate (str, -1, NULL)) {
+		g_free (str);
+		return NULL;
+	}
+
+	return str;
+}
+
 static TotemPlParserResult
 totem_pl_parser_add_iso (TotemPlParser *parser, const char *url,
 		gpointer data)
 {
 	GnomeVFSFileInfo *info;
-	char *item;
+	char *item, *label;
 
 	/* This is a hack, it could be a VCD or DVD */
 	if (g_str_has_prefix (url, "file://") == FALSE)
@@ -1846,7 +1930,10 @@ totem_pl_parser_add_iso (TotemPlParser *parser, const char *url,
 	}
 
 	gnome_vfs_file_info_unref (info);
-	totem_pl_parser_add_one_url (parser, item, NULL);
+
+	label = totem_pl_parser_iso_get_title (url);
+	totem_pl_parser_add_one_url (parser, item, label);
+	g_free (label);
 	g_free (item);
 
 	return TOTEM_PL_PARSER_RESULT_SUCCESS;
@@ -1988,17 +2075,22 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 	MediaType type;
 	GList *list, *l;
 	GnomeVFSResult res;
+	char *media_url;
 
-	if (parser->priv->recurse_level == 1) {
-		char *media_url;
+	type = totem_cd_detect_type_from_dir (url, &media_url, NULL);
+	if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR) {
+		if (media_url != NULL) {
+			char *basename = NULL, *fname;
 
-		type = totem_cd_detect_type_from_dir (url, &media_url, NULL);
-		if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR) {
-			if (media_url != NULL) {
-				totem_pl_parser_add_one_url (parser, media_url, NULL);
-				g_free (media_url);
-				return TOTEM_PL_PARSER_RESULT_SUCCESS;
+			fname = g_filename_from_uri (url, NULL, NULL);
+			if (fname != NULL) {
+				basename = g_filename_display_basename (fname);
+				g_free (fname);
 			}
+			totem_pl_parser_add_one_url (parser, media_url, basename);
+			g_free (basename);
+			g_free (media_url);
+			return TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 	}
 
