@@ -1,6 +1,6 @@
-/* Totem Mozilla plugin
+/* Totem Mozilla Embedded Viewer
  *
- * Copyright (C) <2004-2005> Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) <2004-2006> Bastien Nocera <hadess@hadess.net>
  * Copyright (C) <2002> David A. Schleef <ds@schleef.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -19,6 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+//FIXME damn build system!
+#include "totem-interface.c"
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <stdlib.h>
@@ -27,22 +30,28 @@
 #include <gdk/gdkx.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <totem-pl-parser.h>
 
 #include <dbus/dbus-glib.h>
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+
 #include "bacon-video-widget.h"
 #include "totem-interface.h"
 #include "totem-mozilla-options.h"
 #include "bacon-volume.h"
-//FIXME damn build system!
-#include "totem-interface.c"
 #include "video-utils.h"
 
 GtkWidget *totem_volume_create (void);
 
 #define OPTION_IS(x) (strcmp(argv[i], x) == 0)
+#define IS_FD_STREAM (strcmp(emb->filename, "fd://0") == 0)
+
+#define VOLUME_DOWN_OFFSET -8
+#define VOLUME_UP_OFFSET 8
 
 /* For newer D-Bus version */
 #ifndef DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT
@@ -56,7 +65,12 @@ typedef enum {
 } TotemStates;
 
 #define TOTEM_TYPE_EMBEDDED (totem_embedded_get_type ())
-typedef GObjectClass TotemEmbeddedClass;
+typedef struct {
+	GObjectClass parent_class;
+
+	void (*stop_sending_data) (GObject *emb);
+} TotemEmbeddedClass;
+
 typedef struct _TotemEmbedded {
 	GObject parent;
 
@@ -65,32 +79,71 @@ typedef struct _TotemEmbedded {
 	GtkWidget *about;
 	int width, height;
 	const char *orig_filename;
+	const char *mimetype;
 	char *filename, *href, *target;
 	BaconVideoWidget *bvw;
-	gboolean controller_hidden;
 	TotemStates state;
 	GdkCursor *cursor;
 
+	/* Playlist */
+	gboolean is_playlist;
+	GList *playlist, *current;
+	GMainLoop *loop;
+	int num_items;
+
+	/* Open menu item */
+	GnomeVFSMimeApplication *app;
+	GtkWidget *menu_item;
+
 	/* Seek bits */
 	GtkAdjustment *seekadj;
 	GtkWidget *seek;
-	gboolean seeking;
 
-	/* XEmbed */
-	gboolean embedded_done;
+	guint embedded_done : 1;
+	guint controller_hidden : 1;
+	guint hidden : 1;
+	guint repeat : 1;
+	guint seeking : 1;
+	guint noautostart : 1;
 } TotemEmbedded;
+
 GType totem_embedded_get_type (void);
 
 G_DEFINE_TYPE (TotemEmbedded, totem_embedded, G_TYPE_OBJECT);
-static void totem_embedded_class_init (TotemEmbeddedClass *klass) { }
 static void totem_embedded_init (TotemEmbedded *emb) { }
 
 gboolean totem_embedded_play (TotemEmbedded *emb, GError **err);
 gboolean totem_embedded_pause (TotemEmbedded *emb, GError **err);
 gboolean totem_embedded_stop (TotemEmbedded *emb, GError **err);
+gboolean totem_embedded_set_local_file (TotemEmbedded *emb, const char *url, GError **err);
+
+static void totem_embedded_set_menu (TotemEmbedded *emb, gboolean enable);
+static void on_open1_activate (GtkButton *button, TotemEmbedded *emb);
 
 #include "totem-mozilla-interface.h"
 
+enum {
+	STOP_SENDING_DATA,
+	LAST_SIGNAL
+};
+static int totem_emb_table_signals[LAST_SIGNAL] = { 0 };
+
+static void totem_embedded_class_init (TotemEmbeddedClass *klass)
+{
+	GObjectClass *object_class;
+
+	object_class = (GObjectClass *) klass;
+
+	totem_emb_table_signals[STOP_SENDING_DATA] =
+		g_signal_new ("stop-sending-data",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (TotemEmbeddedClass, stop_sending_data),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
+}
+
 static void
 totem_embedded_exit (TotemEmbedded *emb)
 {
@@ -106,6 +159,17 @@ totem_embedded_error_and_exit (char *title, char *reason, TotemEmbedded *emb)
 	totem_embedded_exit (emb);
 }
 
+static void
+totem_embedded_emit_stop_sending_data (TotemEmbedded *emb)
+{
+	g_return_if_fail (emb->filename != NULL);
+	if (IS_FD_STREAM) {
+		g_signal_emit (G_OBJECT (emb),
+				totem_emb_table_signals[STOP_SENDING_DATA],
+				0, NULL);
+	}
+}
+
 static void
 totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
@@ -136,7 +200,8 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 
 	image = glade_xml_get_widget (emb->xml, "emb_pp_button_image");
 	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
-	gdk_window_set_cursor (GTK_WIDGET (emb->bvw)->window, cursor);
+	if (emb->hidden == FALSE && cursor)
+		gdk_window_set_cursor (GTK_WIDGET (emb->bvw)->window, cursor);
 
 	emb->state = state;
 }
@@ -163,6 +228,8 @@ totem_embedded_open (TotemEmbedded *emb)
 	{
 		char *msg, *disp;
 
+		totem_embedded_emit_stop_sending_data (emb);
+
 		totem_embedded_set_state (emb, STATE_STOPPED);
 
 		//FIXME if emb->filename is fd://0 or stdin:///
@@ -178,13 +245,20 @@ totem_embedded_open (TotemEmbedded *emb)
 		}
 
 		g_free (msg);
-
 		g_error_free (err);
+
+		totem_embedded_set_pp_state (emb, FALSE);
 	} else {
 		totem_embedded_set_state (emb, STATE_PAUSED);
 		totem_embedded_set_pp_state (emb, TRUE);
 	}
 
+	if (retval == FALSE || IS_FD_STREAM) {
+		totem_embedded_set_menu (emb, FALSE);
+	} else {
+		totem_embedded_set_menu (emb, TRUE);
+	}
+
 	return retval;
 }
 
@@ -212,95 +286,113 @@ totem_embedded_stop (TotemEmbedded *emb, GError **err)
 	return TRUE;
 }
 
-static GdkWindow *
-totem_gtk_plug_get_toplevel (GtkPlug *plug)
+gboolean
+totem_embedded_set_local_file (TotemEmbedded *emb,
+			       const char *path, GError **err)
 {
-	Window root, parent, *children;
-	guint nchildren;
-	GdkNativeWindow xid;
+	g_message ("Setting the current path to %s", path);
 
-	g_return_val_if_fail (GTK_IS_PLUG (plug), NULL);
+	g_free (emb->filename);
+	emb->filename = g_filename_to_uri (path, NULL, NULL);
 
-	xid = gtk_plug_get_id (plug);
+	return TRUE;
+}
 
-	do
-	{
-		if (XQueryTree (GDK_DISPLAY (), xid, &root,
-					&parent, &children, &nchildren) == 0)
-		{
-			g_warning ("Couldn't find window manager window");
-			return None;
-		}
+static void
+totem_embedded_set_menu (TotemEmbedded *emb, gboolean enable)
+{
+	GtkWidget *menu, *item, *image;
+	GtkWidget *copy;
+	char *label;
 
-		if (root == parent) {
-			GdkWindow *toplevel;
-			toplevel = gdk_window_foreign_new (xid);
-			return toplevel;
-		}
+	copy = glade_xml_get_widget (emb->menuxml, "copy_location1");
+	gtk_widget_set_sensitive (copy, enable);
 
-		xid = parent;
+	if (emb->menu_item != NULL) {
+		gtk_widget_destroy (emb->menu_item);
+		emb->menu_item = NULL;
 	}
-	while (TRUE);
+	if (emb->app != NULL) {
+		gnome_vfs_mime_application_free (emb->app);
+		emb->app = NULL;
+	}
+
+	if (enable == FALSE)
+		return;
+
+	emb->app = gnome_vfs_mime_get_default_application_for_uri (emb->filename, emb->mimetype);
+	if (emb->app == NULL)
+		return;
+
+	/* translators: this is:
+	 * Open With ApplicationName
+	 * as in nautilus' right-click menu */
+	label = g_strdup_printf ("_Open with \"%s\"", emb->app->name);
+	item = gtk_image_menu_item_new_with_mnemonic (label);
+	g_free (label);
+	image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (on_open1_activate), emb);
+	gtk_widget_show (item);
+	emb->menu_item = item;
+
+	menu = glade_xml_get_widget (emb->menuxml, "menu");
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+}
+
+static void
+on_open1_activate (GtkButton *button, TotemEmbedded *emb)
+{
+	GList *l = NULL;
+
+	g_return_if_fail (emb->app != NULL);
+
+	l = g_list_prepend (l, emb->filename);
+	if (gnome_vfs_mime_application_launch (emb->app, l) == GNOME_VFS_OK) {
+		totem_embedded_stop (emb, NULL);
+	}
+
+	g_list_free (l);
 }
 
 static void
 on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 {
-	GdkPixbuf *pixbuf = NULL;
+	char *backend_version, *description;
+
 	const char *authors[] =
 	{
 		"Bastien Nocera <hadess@hadess.net>",
 		"Ronald Bultje <rbultje@ronald.bitfreak.net>",
 		NULL
 	};
-	char *backend_version, *description;
-	char *filename;
-
+	
 	if (emb->about != NULL)
 	{
 		gtk_window_present (GTK_WINDOW (emb->about));
 		return;
 	}
 
-	filename = g_build_filename (DATADIR,
-			"totem", "media-player-48.png", NULL);
-	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-	g_free (filename);
-
 	backend_version = bacon_video_widget_get_backend_name (emb->bvw);
 	description = g_strdup_printf (_("Movie Player using %s"),
-				backend_version);
+				       backend_version);
 
 	emb->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
-			"name", _("Totem Mozilla Plugin"),
-			"version", VERSION,
-			"copyright", _("Copyright \xc2\xa9 2002-2005 Bastien Nocera"),
-			"comments", description,
-			"authors", authors,
-			"translator-credits", _("translator-credits"),
-			"logo", pixbuf,
-			NULL);
+				   "name", _("Totem Mozilla Plugin"),
+				   "version", VERSION,
+				   "copyright", _("Copyright \xc2\xa9 2002-2006 Bastien Nocera"),
+				   "comments", description,
+				   "authors", authors,
+				   "translator-credits", _("translator-credits"),
+				   "logo-icon-name", "totem",
+				   NULL);
 
 	g_free (backend_version);
 	g_free (description);
 
-	if (GTK_IS_PLUG (emb->window)) {
-		GdkWindow *toplevel;
-
-		gtk_widget_realize (emb->about);
-		toplevel = totem_gtk_plug_get_toplevel (GTK_PLUG (emb->window));
-		if (toplevel != NULL) {
-			gdk_window_set_transient_for
-				(GTK_WIDGET (emb->about)->window, toplevel);
-			gdk_window_unref (toplevel);
-		}
-	} else {
-		gtk_window_set_transient_for (GTK_WINDOW (emb->about),
-				GTK_WINDOW (emb->window));
-	}
-
-	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
+	totem_interface_set_transient_for (GTK_WINDOW (emb->about),
+			GTK_WINDOW (emb->window));
 
 	g_object_add_weak_pointer (G_OBJECT (emb->about),
 			(gpointer *)&emb->about);
@@ -310,6 +402,20 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	gtk_widget_show (emb->about);
 }
 
+static void
+on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb)
+{
+	GtkClipboard *clip;
+
+	/* Set both the middle-click and the super-paste buffers */
+	clip = gtk_clipboard_get_for_display
+		(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clip, emb->filename, -1);
+	clip = gtk_clipboard_get_for_display
+		(gdk_display_get_default(), GDK_SELECTION_PRIMARY);
+	gtk_clipboard_set_text (clip, emb->filename, -1);
+}
+
 static void
 on_play_pause (GtkWidget *widget, TotemEmbedded *emb)
 {
@@ -408,8 +514,45 @@ on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
 {
 	totem_embedded_set_state (emb, STATE_STOPPED);
 	gtk_adjustment_set_value (emb->seekadj, 0);
-	if (strcmp (emb->filename, "fd://0") == 0) {
+
+	/* No playlist if we have fd://0, right? */
+	if (IS_FD_STREAM) {
 		totem_embedded_set_pp_state (emb, FALSE);
+	} else if (emb->num_items == 1) {
+		if (g_str_has_prefix (emb->filename, "file://") != FALSE) {
+			if (bacon_video_widget_is_seekable (emb->bvw) != FALSE) {
+				bacon_video_widget_pause (emb->bvw);
+				bacon_video_widget_seek (emb->bvw, 0.0, NULL);
+			} else {
+				bacon_video_widget_close (emb->bvw);
+				totem_embedded_open (emb);
+			}
+		} else {
+			bacon_video_widget_close (emb->bvw);
+			totem_embedded_open (emb);
+		}
+		if (emb->repeat != FALSE && emb->noautostart == FALSE)
+			totem_embedded_play (emb, NULL);
+	} else {
+		/* Multiple items on the playlist */
+		gboolean eop = FALSE, res;
+
+		if (emb->current->next == NULL) {
+			emb->current = emb->playlist;
+			eop = TRUE;
+		} else {
+			emb->current = emb->current->next;
+		}
+
+		g_free (emb->filename);
+		emb->filename = g_strdup (emb->current->data);
+		bacon_video_widget_close (emb->bvw);
+		res = totem_embedded_open (emb);
+		if (res != FALSE &&
+				((eop != FALSE && emb->repeat != FALSE)
+				 || (eop == FALSE))) {
+			totem_embedded_play (emb, NULL);
+		}
 	}
 }
 
@@ -420,6 +563,31 @@ cb_vol (GtkWidget *val, TotemEmbedded *emb)
 		bacon_volume_button_get_value (BACON_VOLUME_BUTTON (val)));
 }
 
+static gboolean
+on_volume_scroll_event (GtkWidget *win, GdkEventScroll *event, TotemEmbedded *emb)
+{
+	GtkWidget *vbut;
+	int vol, offset;
+
+	switch (event->direction) {
+	case GDK_SCROLL_UP:
+		offset = VOLUME_UP_OFFSET;
+		break;
+	case GDK_SCROLL_DOWN:
+		offset = VOLUME_DOWN_OFFSET;
+		break;
+	default:
+		return FALSE;
+	}
+
+	vbut = glade_xml_get_widget (emb->xml, "volume_button");
+	vol = bacon_volume_button_get_value (BACON_VOLUME_BUTTON (vbut));
+	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut),
+			vol + offset);
+
+	return FALSE;
+}
+
 static void
 on_tick (GtkWidget *bvw,
 		gint64 current_time,
@@ -458,6 +626,7 @@ static void
 totem_embedded_add_children (TotemEmbedded *emb)
 {
 	GtkWidget *child, *container, *pp_button, *vbut;
+	BvwUseType type = BVW_USE_TYPE_VIDEO;
 	GError *err = NULL;
 	GConfClient *gc;
 	int volume;
@@ -470,15 +639,16 @@ totem_embedded_add_children (TotemEmbedded *emb)
 			GTK_WINDOW (emb->window));
 
 	if (emb->xml == NULL || emb->menuxml == NULL)
-	{
 		totem_embedded_exit (emb);
-	}
 
 	child = glade_xml_get_widget (emb->xml, "vbox1");
 	gtk_container_add (GTK_CONTAINER (emb->window), child);
 
+	if (emb->hidden)
+		type = BVW_USE_TYPE_AUDIO;
+
 	emb->bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new
-			(-1, -1, BVW_USE_TYPE_VIDEO, &err));
+			(-1, -1, type, &err));
 
 	if (emb->bvw == NULL)
 	{
@@ -498,8 +668,10 @@ totem_embedded_add_children (TotemEmbedded *emb)
 
 	container = glade_xml_get_widget (emb->xml, "hbox4");
 	gtk_container_add (GTK_CONTAINER (container), GTK_WIDGET (emb->bvw));
-	gtk_widget_realize (GTK_WIDGET (emb->bvw));
-	gtk_widget_show (GTK_WIDGET (emb->bvw));
+	if (type == BVW_USE_TYPE_VIDEO) {
+		gtk_widget_realize (GTK_WIDGET (emb->bvw));
+		gtk_widget_show (GTK_WIDGET (emb->bvw));
+	}
 
 	emb->seek = glade_xml_get_widget (emb->xml, "time_hscale");
 	emb->seekadj = gtk_range_get_adjustment (GTK_RANGE (emb->seek));
@@ -521,6 +693,9 @@ totem_embedded_add_children (TotemEmbedded *emb)
 	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut), volume);
 	g_signal_connect (G_OBJECT (vbut), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
+	gtk_widget_add_events (vbut, GDK_SCROLL_MASK);
+	g_signal_connect (G_OBJECT (vbut), "scroll_event",
+			G_CALLBACK (on_volume_scroll_event), emb);
 
 	gtk_widget_realize (emb->window);
 	gtk_widget_set_size_request (emb->window, emb->width, emb->height);
@@ -534,6 +709,11 @@ totem_embedded_add_children (TotemEmbedded *emb)
 	child = glade_xml_get_widget (emb->menuxml, "about1");
 	g_signal_connect (G_OBJECT (child), "activate",
 			  G_CALLBACK (on_about1_activate), emb);
+	child = glade_xml_get_widget (emb->menuxml, "copy_location1");
+	g_signal_connect (G_OBJECT (child), "activate",
+			G_CALLBACK (on_copy_location1_activate), emb);
+	child = glade_xml_get_widget (emb->menuxml, "preferences1");
+	gtk_widget_hide (child);
 }
 
 static void
@@ -553,6 +733,70 @@ totem_embedded_create_cursor (TotemEmbedded *emb)
 	gdk_pixbuf_unref (icon);
 }
 
+static void
+entry_added (TotemPlParser *parser, const char *uri, const char *title,
+		                const char *genre, gpointer data)
+{
+	TotemEmbedded *emb = (TotemEmbedded *) data;
+
+	g_print ("added URI '%s' with title '%s' genre '%s'\n", uri,
+			title ? title : "empty", genre);
+
+	//FIXME need new struct to hold that
+	emb->playlist = g_list_prepend (emb->playlist, g_strdup (uri));
+	emb->num_items++;
+}
+
+static gboolean
+totem_embedded_push_parser (gpointer data)
+{
+	TotemPlParser *parser = totem_pl_parser_new ();
+	TotemEmbedded *emb = (TotemEmbedded *) data;
+	TotemPlParserResult res;
+
+	parser = totem_pl_parser_new ();
+	g_signal_connect (G_OBJECT (parser), "entry", G_CALLBACK (entry_added), emb);
+	res = totem_pl_parser_parse (parser, emb->filename, FALSE);
+	g_object_unref (parser);
+
+	if (res != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+		//FIXME show a proper error message
+		switch (res) {
+		case TOTEM_PL_PARSER_RESULT_UNHANDLED:
+			g_print ("url '%s' unhandled\n", emb->filename);
+			break;
+		case TOTEM_PL_PARSER_RESULT_ERROR:
+			g_print ("error handling url '%s'\n", emb->filename);
+			break;
+		case TOTEM_PL_PARSER_RESULT_IGNORED:
+			g_print ("ignored url '%s'\n", emb->filename);
+			break;
+		default:
+			g_assert_not_reached ();
+			;;
+		}
+	}
+
+	/* Check if we have anything in the playlist now */
+	if (emb->playlist == NULL) {
+		g_message ("NO PLAYLIST");
+		totem_embedded_error_and_exit ("Can't parse that",
+				"no files",
+				emb);
+		//FIXME error out
+		return FALSE;
+	}
+
+	emb->playlist = g_list_reverse (emb->playlist);
+	g_free (emb->filename);
+	emb->filename = g_strdup (emb->playlist->data);
+	emb->current = emb->playlist;
+
+	g_main_loop_quit (emb->loop);
+
+	return FALSE;
+}
+
 static void embedded (GtkPlug *plug, TotemEmbedded *emb)
 {
 	emb->embedded_done = TRUE;
@@ -572,7 +816,6 @@ totem_volume_create (void)
 
 int main (int argc, char **argv)
 {
-	gboolean window_given = FALSE;
 	TotemEmbedded *emb;
 	DBusGProxy *proxy;
 	DBusGConnection *conn;
@@ -582,6 +825,10 @@ int main (int argc, char **argv)
 	gchar *svcname;
 	GError *e = NULL;
 
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
@@ -592,6 +839,7 @@ int main (int argc, char **argv)
 	bacon_video_widget_init_backend (NULL, NULL);
 
 	gtk_init (&argc, &argv);
+	gnome_vfs_init ();
 	dbus_g_object_type_install_info (TOTEM_TYPE_EMBEDDED,
 		&dbus_glib_totem_embedded_object_info);
 	svcname = g_strdup_printf ("org.totem_%d.MozillaPluginService",
@@ -628,7 +876,6 @@ int main (int argc, char **argv)
 	/* TODO Add popt options */
 	for (i = 1; i < argc; i++) {
 		if (OPTION_IS (TOTEM_OPTION_XID)) {
-			window_given = TRUE;
 			if (i + 1 < argc) {
 				i++;
 				xid = (Window) g_ascii_strtoull (argv[i], NULL, 10);
@@ -660,6 +907,19 @@ int main (int argc, char **argv)
 				i++;
 				emb->target = g_strdup (argv[i]);
 			}
+		} else if (OPTION_IS (TOTEM_OPTION_MIMETYPE)) {
+			if (i + 1 < argc) {
+				i++;
+				emb->mimetype = (const char *) argv[i];
+			}
+		} else if (OPTION_IS (TOTEM_OPTION_HIDDEN)) {
+			emb->hidden = TRUE;
+		} else if (OPTION_IS (TOTEM_OPTION_PLAYLIST)) {
+			emb->is_playlist = TRUE;
+		} else if (OPTION_IS (TOTEM_OPTION_REPEAT)) {
+			emb->repeat = TRUE;
+		} else if (OPTION_IS (TOTEM_OPTION_NOAUTOSTART)) {
+			emb->noautostart = TRUE;
 		} else if (i + 1 == argc) {
 			emb->filename = g_strdup (argv[i]);
 		}
@@ -683,13 +943,11 @@ int main (int argc, char **argv)
 		emb->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	}
 
+	/* Hidden or not as well? */
 	totem_embedded_add_children (emb);
 	totem_embedded_create_cursor (emb);
-	if (!window_given || xid != 0)
+	if (emb->hidden == FALSE)
 		gtk_widget_show (emb->window);
-	else {
-		gtk_widget_realize (emb->window);
-	}
 
 	/* wait until we're embedded if we're to be, or shown */
 	if (xid != 0) {
@@ -700,8 +958,22 @@ int main (int argc, char **argv)
 			gtk_main_iteration ();
 	}
 
-	if (totem_embedded_open (emb) != FALSE)
+	/* Do we have a playlist we need to setup ourselves? */
+	if (emb->is_playlist != FALSE) {
+		g_idle_add (totem_embedded_push_parser, emb);
+		emb->loop = g_main_loop_new (NULL, FALSE);
+		g_main_loop_run (emb->loop);
+		g_main_loop_unref (emb->loop);
+	} else {
+		emb->playlist = g_list_prepend (emb->playlist,
+				g_strdup (emb->filename));
+		emb->num_items++;
+	}
+
+	if (totem_embedded_open (emb) != FALSE
+			&& emb->noautostart == FALSE) {
 		totem_embedded_play (emb, NULL);
+	}
 
 	gtk_main ();
 
